#include <xc.h>
#include <stdint.h>
#include "Main.h"
#include "IIC.h"

volatile	uint8_t		bIIC_OK;	// 5-10 mS time out counter

//************************************************
//* Initial I2C Master Mode with 7 bits Address  *
//* Clock Speed : 100KHz @32MHz					 *
//************************************************
void Initialize_I2C_Master(void)
{
		SSPEN = 0;			// Disable I2C
 		SCL=1;				// Initial PortC,bit 3 & 4 as Input with Open Drain
		SDA=1;				// RC3 = SCL  ,  RC4 = SDA
		SCL_DIR =1;			 
		SDA_DIR =1;

		SSPCON =0b00101000;	// Enable SSP, I2C Master Mode, Clock Rate: FOSC/(4*SSPADD+1)
 		SSPSTAT=0b10000000; // Disable slew rate for 100KHz,and clear status bit
     	SSPCON2=0b00000000;	// Set SCL,SDA into Ready status
 	   	SSPCON3=0b00000000;  
		SSPADD= 79;			// (32MHz/4) / (79+1)= 100KHz  
//		SSPADD= 19;			// (32MHz/4) / (19+1)= 400KHz  
		SSPMSK= 0xFF;
		SSPEN = 1;			// Enable I2C
}

//********************************************************************
// As below are I2C function
// These functions were modified from C18 I2C Libyaries' Source Code
//********************************************************************
//**** Test and wait until I2C module is idle.
uint8_t IdleI2C( void ) 				
{
  	while ( (( SSPCON2 & 0x1F ) || ( R_nW )) && bIIC_OK )
    	continue;

	return bIIC_OK;
}

uint8_t I2C_Done(void)
{
	while (!SSPIF && bIIC_OK); // Completed the action when the SSPIF is Hi.
	if (bIIC_OK)
		SSPIF=0;   		 	// Clear SSPIF
	else {
		SSPEN = 0;			// Disable I2C
		SCL_DIR = 0;			 
		bIIC_OK = 8;
		while ( bIIC_OK )
 		 	SCL=0;			// Pull CLK low at least 35-40 mS
		SCL_DIR = 1;			 
		SSPEN = 1;			// Enable I2C
	}
	return bIIC_OK;
}


//Send I2C bus start condition.
uint8_t StartI2C( void )
{
	bIIC_OK = 2;				// IIC timeout = 5-10 mS

    if ( !IdleI2C() ) return 0; // ensure module is idle
	SEN = 1;            		// initiate bus start condition
	return I2C_Done();		  	// Wait Start condition completed
}

//This routine writes a single byte to the I2C bus.
uint8_t WriteI2C( uint8_t data_out )	
{
  SSPBUF = data_out;           	// write single byte to SSPBUF
  if (WCOL) {
  	WCOL = 0;
	bIIC_OK = 0;
	}

  else
  {
    while(BF && bIIC_OK);   	// wait until write cycle is complete         
    if ( !IdleI2C() ) return 0; // ensure module is idle
    if ( SSPCON2bits.ACKSTAT ) 	// test for ACK condition received
		bIIC_OK = 0;
	else
		return	I2C_Done();		// Clear SSPIF flag
  }
  return bIIC_OK;
}

uint8_t ReadI2C( uint8_t *bData )
{
  RCEN = 1;           			// enable master for 1 byte reception
  while ( !BF && bIIC_OK);      // wait until byte received 
  if ( !bIIC_OK )	return 0;
  *bData = SSPBUF;            	// return with read byte
  return I2C_Done();			// Clear SSPIF flag
}

uint8_t AckI2C( void )
{
  ACKDT = 0;           // set acknowledge bit state for ACK
  ACKEN = 1;           // initiate bus acknowledge sequence
  return I2C_Done();   // Clear SSPIF flag
}

uint8_t NotAckI2C( void )
{
  ACKDT = 1;          // set acknowledge bit for not ACK
  ACKEN = 1;          // initiate bus acknowledge sequence
  return I2C_Done();  // Clear SSPIF flag
}

uint8_t RestartI2C( void )
{
  RSEN = 1;           // initiate bus restart condition
  return I2C_Done();  // Clear SSPIF flag
}
uint8_t StopI2C( void )
{
  PEN = 1;            // initiate bus stop condition
  return I2C_Done();  // wait until stop condition is over 
}


/*******************************************************************************
	Issuse a read block command	

	Input: bCMData = Target register Address for Reading
	output: Byte date of the UCS1002's register : [bCMData]
*******************************************************************************/
uint8_t ReadBlock( uint8_t bSlaveAddr,	uint8_t bCMData, uint8_t *bBuf, uint8_t bCnt ) 
{
	uint8_t	bDCnt=bCnt, bRetry;
	uint8_t	*bPtr = bBuf;
	
	for ( bRetry = 3; bRetry ; --bRetry )
	{
  		if ( StartI2C() && 				// Start condition
		     WriteI2C(bSlaveAddr) &&	// Write slave address 
		     WriteI2C( bCMData) &&		// Write CMD. 
   			 RestartI2C() && 			// initiate Restart condition
		     WriteI2C(bSlaveAddr | 0x01 ) // Read from slave address	  
		   )
		{
			
			do {
				if ( !ReadI2C( bBuf++ ) ) // Enable I2C Receiver & wait BF=1 until received data
					break;
			} while ( --bCnt && AckI2C( ) );



			if ( !bCnt && NotAckI2C() 	// Genarate Non_Acknowledge 
            		   && StopI2C()     // send STOP condition
			   )
			{
//				LED8_Off;				// IIC OK
				break;
			}
		}
		
		bCnt = bDCnt,	bBuf = bPtr;
//		LED8_On;						// IIC Error
		Initialize_I2C_Master();		// Re-init. IIC
	}
	return	bDCnt-bCnt;					// total bytes read
}

/*******************************************************************************
	Issuse a read byte command to UCS1002 via SMBus	

	Input: bCMData = Target register Address for Reading
	output: Byte date of the UCS1002's register : [bCMData]
*******************************************************************************/
uint8_t ReadByte( uint8_t bSlaveAddr,	uint8_t bCMData ) 
{
//	uint8_t	bData = 0, bRetry;
//	
//	for ( bRetry = 3; bRetry ; --bRetry )
//	{
//  		if ( StartI2C() && 				// Start condition
//		     WriteI2C(bSlaveAddr) &&	// Write slave address 
//		     WriteI2C( bCMData) &&		// Write CMD. 
//   			 RestartI2C() && 			// initiate Restart condition
//		     WriteI2C(bSlaveAddr | 0x01 ) && // Read from slave address	  
//		     ReadI2C( &bData) &&		// Enable I2C Receiver & wait BF=1 until received data
//			 NotAckI2C() &&				// Genarate Non_Acknowledge 
//             StopI2C()                  // send STOP condition
//		   	)
//			{
//				LED8_Off;				// IIC OK
//				break;
//			}
//		LED8_On;						// IIC Error
//		Initialize_I2C_Master();		// Re-init. IIC
//	}
//	return	bData;
	uint8_t	bData;

	if ( ReadBlock( bSlaveAddr,	bCMData, &bData, 1 ) == 1 )
		return bData;
	return 0;
}

/*******************************************************************************
	Issuse a write byte command to UCS1002 via SMBus	

	Input: bCMData = Target register Address for writing
		   bData = output data for writing to UCS1002
	output: NONE
*******************************************************************************/
void WriteByte( uint8_t bSlaveAddr, uint8_t bCMData, uint8_t bData ) 
{
	uint8_t	bRetry;

	for ( bRetry = 3; bRetry ; --bRetry )
	{
  		if ( StartI2C() && 			// Start condition
		     WriteI2C( bSlaveAddr ) && // Write slave address 
		     WriteI2C( bCMData) &&	// Write CMD.
			 WriteI2C( bData) && 	// Write Data
             StopI2C()              // send STOP condition
		    )
		{
//			LED8_Off;			// IIC OK
			break;
		}
//		LED8_On;					// IIC Error

		Initialize_I2C_Master();	// Re-Init. IIC
	}
}


